# Copyright (C) 2010 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from bzrlib import branch, commands, errors, option
from bzrlib.plugins.difftacular import generate_diff

class cmd_diff_mainline(commands.Command):

    takes_options = ['revision']

    def run(self, revision=None):
        my_branch = branch.Branch.open_containing('.')[0]
        my_branch.lock_read()
        try:
            if revision is None or len(revision) < 1:
                raise errors.BzrCommandError(
                    'One or more revisions must be supplied.')
            old_revision = revision[0].as_revision_id(my_branch)
            if len(revision) > 1:
                new_revision = revision[1].as_revision_id(my_branch)
            else:
                new_revision = my_branch.last_revision()
            generate_diff.mainline_diff(my_branch, old_revision, new_revision,
                                        self.outf)
        finally:
            my_branch.unlock()


class cmd_diff_ignore_branches(commands.Command):

    takes_options = ['revision']
    takes_args = ['branch*']

    def run(self, revision=None, branch_list=None):
        if branch_list is None:
            branch_list = []
        my_branch = branch.Branch.open_containing('.')[0]
        my_branch.lock_read()
        try:
            if revision is None or len(revision) < 1:
                raise errors.BzrCommandError(
                    'One or more revisions must be supplied.')
            old_revision = revision[0].as_revision_id(my_branch)
            if len(revision) > 1:
                new_revision = revision[1].as_revision_id(my_branch)
            else:
                new_revision = my_branch.last_revision()
            branches = [branch.Branch.open_containing(b)[0]
                        for b in branch_list]
            writer = option.diff_writer_registry.get()
            generate_diff.diff_ignore_branches(
                my_branch, branches, old_revision, new_revision,
                writer(self.outf))
        finally:
            my_branch.unlock()
